# areacalc.sml
# Sample script for Building Dialogs in SML

# The "Area Unit Conversion" window accepts 
# numeric input (an input area), uses option 
# menus to allow the user to select input and 
# output area units, and computes an output value.

################################################
# Declare class instances used in procedures
################################################
class GUI_CTRL_EDIT_NUMBER inArea;		# class instance for the input area control

class GUI_CTRL_EDIT_STRING outAreaStr;	# class instance for control showing output area

class GUI_DLG dlgwin;						# class instance for dialog window

################################################
# Declare global variables
################################################
string xmlfile$;
numeric errXML;

numeric userval;
string inAreaUnit$ = "square meters"; 
string outAreaUnit$ = "square meters";

numeric dlgreturn;

################################################
# Define procedures and functions
################################################

# function to calculate and show the new output area value.
func Recalculate () {
	local numeric scale;		# scale for unit conversion
	local numeric outarea;	# computed output area value
	local string area$;		# string for computed area with units

	scale = GetUnitConvArea( inAreaUnit$, outAreaUnit$ );
	printf( "conversion factor= %.9f\n", scale );
	outarea = userval * scale;
	printf( "Output area= %.9f\n", outarea );
	area$ = sprintf( "%.9f %s", outarea, outAreaUnit$ );

	dlgwin.SetCtrlValueStr("outAreaStr", area$);
	return 0;
	}

proc OnNewValue () {

	inArea = dlgwin.GetCtrlByID( "inArea" );
	userval = inArea.GetValueNum();
	printf( "Input area= %10.2f\n", userval );
	outAreaStr.ClearValue(0);
	}

proc OnInputUnitChange () {

	inAreaUnit$ = dlgwin.GetCtrlValueStr( "inUnits" );
	printf("%s\n",inAreaUnit$);
	outAreaStr.ClearValue(0);
	}

proc OnOutputUnitChange () {

	outAreaUnit$ = dlgwin.GetCtrlValueStr( "outUnits" );
	printf( "Output units= %s\n", outAreaUnit$ );
	outAreaStr.ClearValue(0);
	}

###################################################
# Main program
###################################################
clear();

xmlfile$ = _context.ScriptDir + "/areacalc.xml";


class XMLDOC dlgdoc;		# class instance for the XML document
errXML = dlgdoc.Read(xmlfile$);		# read the XML and parse into memory

if (errXML < 0 ) {
	PopupError(errXML);
	Exit();
	}

class XMLNODE dlgnode;	# class instance for the dialog element
								# in the XML structure

dlgnode = dlgdoc.GetElementByID("areacalc");	# get the dialog element
													# from the XML structure

if (dlgnode == 0) {
	PopupMessage("Could not find dialog node in XML document");
	Exit();
	}


dlgwin.SetXMLNode(dlgnode);	# set the XML dialog element as the
										# source for the dialog window

dlgreturn = dlgwin.DoModal();					# open as a modal dialog

printf("dlgreturn= %d\n", dlgreturn);

if (dlgreturn == -1) {		# exit script when Cancel button on dialog is pressed
	Exit();
	}


